import pytest

import mutf8.mutf8 as pymutf8
import mutf8.cmutf8 as cmutf8


def pytest_addoption(parser):
    parser.addoption(
        '--runslow',
        action='store_true',
        default=False,
        help='run slow tests'
    )


def pytest_configure(config):
    config.addinivalue_line('markers', 'slow: mark test as slow to run')


def pytest_collection_modifyitems(config, items):
    if config.getoption('--runslow'):
        # --runslow given in cli: do not skip slow tests
        return

    skip_slow = pytest.mark.skip(reason='need --runslow option to run')
    for item in items:
        if 'slow' in item.keywords:
            item.add_marker(skip_slow)


@pytest.fixture(params=[pymutf8, cmutf8])
def module(request):
    yield request.param


@pytest.fixture()
def decoder(module):
    yield module.decode_modified_utf8


@pytest.fixture()
def encoder(module):
    yield module.encode_modified_utf8
